﻿using Microsoft.EntityFrameworkCore;
using System.Collections.Generic;

namespace UserService.DatabaseLayer.DataModels
{
    public static class ModelBuilderExtensions
    {
        public static void Seed(this ModelBuilder modelBuilder)
        {
            var groups = new OrganizationUnit { CommonName = "Groups", Id = -1 };
            var users = new OrganizationUnit { CommonName = "Users", Id = -2 };
            var germany = new OrganizationUnit{CommonName = "Germany", Id = -6, ParentId = -2};
            var usa = new OrganizationUnit{CommonName = "USA", Id = -5, ParentId = -2};
            var arizona = new OrganizationUnit{CommonName = "Arizona" , Id = -4, ParentId = -5 };
            var france = new OrganizationUnit{CommonName = "France" , Id = -3, ParentId = -2 };
            modelBuilder.Entity<OrganizationUnit>().HasData(users, groups, germany, usa, arizona, france);
            var user = new User { CommonName = "holger", IsActive = true, Id = -7, ParentId = germany.Id };
            modelBuilder.Entity<User>().HasData(user);
            var secGroup = new SecurityGroup { CommonName = "Global Admin", Id = -8, ParentId = groups.Id };
            modelBuilder.Entity<SecurityGroup>().HasData(secGroup);

            modelBuilder.Entity<UserMember>()
                .HasData(new UserMember { MemberId = secGroup.Id, UserId = user.Id });
        }

        public static void CreateRelations(this ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<UserMember>()
                .HasKey(bc => new { bc.MemberId, bc.UserId });
            modelBuilder.Entity<UserMember>()
                .HasOne(bc => bc.User)
                .WithMany(b => b!.MemberOf)
                .HasForeignKey(bc => bc.UserId);
            modelBuilder.Entity<UserMember>()
                .HasOne(bc => bc.Member)
                .WithMany(c => c!.Members)
                .HasForeignKey(bc => bc.MemberId);

            modelBuilder.Entity<Node>()
                .HasMany(c => c.Children)
                .WithOne(e => e.Parent!)
                .HasForeignKey(bc => bc.ParentId);
        }
    }

    public static class UserExtensions
    {
        public static IEnumerable<SecurityGroup> GetSecurityGroups(this User user)
        {
            foreach (var userMember in user.MemberOf)
            {
                if (userMember.Member is SecurityGroup securityGroup)
                {
                    yield return securityGroup;
                }
            }
        }

    }

    public static class SecurityGroupExtensions
    {
        public static IEnumerable<User> GetUsers(this SecurityGroup securityGroup)
        {
            foreach (var userMember in securityGroup.Members)
            {
                if (userMember.User is null) continue;
                yield return userMember.User;
            }
        }

    }
}